use rustible::Facts;

use rustible::modules::{
    Module, PlaybookRun,
    package::{PackageModule as Package, PackageState, PackageError},
    command::{CommandModule as Command, CommandError},
};

#[derive(Clone, Debug, serde::Serialize)]
pub enum PlaybookError {
    Command(CommandError),
    Package(PackageError),
}

impl From<CommandError> for PlaybookError {
    fn from(e: CommandError) -> PlaybookError {
        PlaybookError::Command(e)
    }
}

impl From<PackageError> for PlaybookError {
    fn from(e: PackageError) -> PlaybookError {
        PlaybookError::Package(e)
    }
}

fn main() -> Result<(), PlaybookError> {
    let facts = Facts::new();
    println!("Hello, world! Running system {}", facts.os.family().as_str());

    let mut playbook = PlaybookRun::new();

    let cmd = Command::new().program("echo").args(&vec!("lol".to_string())).build();
    let res = cmd.run()?;
    println!("{}", res.success());
    
    let cmd = Command::new().program("echo").args(&vec!("lol".to_string())).build();
    playbook.run(cmd)?;
    
    let cmd = Command::new()
        .creates("/tmp/lol")
        .program("touch")
        .arg("/tmp/lol")
        .build();
    playbook.run(cmd)?;

    let pkg = Package::new()
        .name("hello")
        .state(PackageState::Present)
        .build();
    playbook.run(pkg)?;

    let pkg = Package::new()
        .name("hello")
        .state(PackageState::Absent)
        .build();
    playbook.run(pkg)?;

    let pkg = Package::new()
        .name(vec!("hello", "sl"))
        .state(PackageState::Present)
        .build();
    playbook.run(pkg)?;

    let pkg = Package::new()
        .name(&[ "hello", "sl" ])
        .state(PackageState::Absent)
        .build();
    playbook.run(pkg)?;
    playbook.print_json_pretty();

    Ok(())
}